@@ -34,6 +34,8 @@
 
 #include <vector>
 
+#include <sys/types.h>
+
 #include <openssl/rsa.h>
 
 #include "slog.h"
@@ -93,6 +95,7 @@ struct conf_node
   enum connectmode { C_ONDEMAND, C_NEVER, C_ALWAYS, C_DISABLED } connectmode;
   bool compress;
   bool inherit_tos; // inherit TOS in packets send to this destination
+  bool low_power; // node is a low-power node (reduce cpu usage, wakeups and assume higher latency)
 
   vector<const char *> allow_direct;
   vector<const char *> deny_direct;
@@ -125,6 +128,7 @@ struct configuration
   node_vector nodes;
   conf_node default_node;
   conf_node *thisnode;
+  char serial[SERIAL_SIZE];
   char *seed_dev;   // the randomd evice to use for seeding
   double reseed;    // the interval between additional seeds
   int mtu;          // the mtu used for outgoing tunnel packets
@@ -170,6 +174,8 @@ struct configuration
   void cleanup ();
   void clear ();
 
+  conf_node *find_node (const char *name);
+
   // create a filename from string, replacing %s by the nodename
   // and using relative paths under confbase.
   char *config_filename (const char *name, const char *dflt = 0);
