@@ -63,10 +63,7 @@ struct sockinfo
 
   const char *ntoa () const;
 
-  bool valid () const
-  {
-    return prot != 0 && host != 0;
-  }
+  bool valid () const;
 
   sockinfo() { prot = 0; }
 
